@@ -30,15 +30,18 @@ class Command:
         self.api.teleop_enable(value)
         self.api.wait_complete()
 
+    def set_motion_teleop(self, value):
+        self.api.teleop_enable(value)
+        self.api.wait_complete(0.1)
+        self.father.status.api.poll()
+
     def set_teleop_enable_mode(self, value):
         self.father.status.api.poll()
         if self.father.status.api.task_mode != linuxcnc.MODE_MANUAL:
             self.set_mode(linuxcnc.MODE_MANUAL, 1)
         if self.get_jog_mode():
             return
-        self.api.teleop_enable(value)
-        self.api.wait_complete(0.1)
-        self.father.status.api.poll()
+        self.set_motion_teleop(value)
         return True
 
     def get_jog_mode(self):
@@ -112,10 +115,22 @@ class Command:
             self.home_axis(x)
 
     def home_axis(self, axis):
-        if self.father.framework.machine.task_state:
+        self.set_teleop_enable_mode(0)
+        if self.father.status.api.task_mode != linuxcnc.MODE_MANUAL:
             self.set_mode(linuxcnc.MODE_MANUAL, 1)
-            self.api.home(axis)
-            self.api.wait_complete()
+        self.api.home(axis)
+        self.api.wait_complete()
+
+    def un_home_all(self):
+        for x in range(len(self.father.framework.machine.axis) - 1, -1, -1):
+            self.un_home_axis(x)
+
+    def un_home_axis(self, axis):
+        self.father.status.api.poll()
+        if self.father.status.api.task_mode != linuxcnc.MODE_MANUAL:
+            self.set_mode(linuxcnc.MODE_MANUAL, 1)
+        self.set_motion_teleop(0)
+        self.api.unhome(axis)
 
     def is_homed(self):
         axes = len(self.father.framework.machine.axis)
